@@ -13,7 +13,7 @@ import sys
 from collections import defaultdict
 from collections.abc import ItemsView, Sequence
 from functools import cached_property
-from typing import TYPE_CHECKING, Any, Dict, List, Union
+from typing import TYPE_CHECKING, Any, Union
 
 import astroid
 from astroid import nodes
@@ -43,7 +43,7 @@ if TYPE_CHECKING:
 
 # The dictionary with Any should actually be a _ImportTree again
 # but mypy doesn't support recursive types yet
-_ImportTree = Dict[str, Union[List[Dict[str, Any]], List[str]]]
+_ImportTree = dict[str, Union[list[dict[str, Any]], list[str]]]
 
 DEPRECATED_MODULES = {
     (0, 0, 0): {"tkinter.tix", "fpectl"},
@@ -79,6 +79,7 @@ DEPRECATED_MODULES = {
         "uu",
         "xdrlib",
     },
+    (3, 13, 0): {"getopt"},
 }
 
 
