@@ -1,17 +1,18 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
-from typing import Union
+# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt
+
+from __future__ import annotations
 
 import astroid
 from astroid import nodes
 
-from pylint import checkers, interfaces
+from pylint import checkers
 from pylint.checkers import utils
+from pylint.interfaces import HIGH, INFERENCE
 
 
 class RecommendationChecker(checkers.BaseChecker):
-
-    __implements__ = (interfaces.IAstroidChecker,)
     name = "refactoring"
     msgs = {
         "C0200": (
@@ -53,21 +54,28 @@ class RecommendationChecker(checkers.BaseChecker):
             "are more efficient than ``sets``.",
         ),
         "C0209": (
-            "Formatting a regular string which could be a f-string",
+            "Formatting a regular string which could be an f-string",
             "consider-using-f-string",
             "Used when we detect a string that is being formatted with format() or % "
-            "which could potentially be a f-string. The use of f-strings is preferred.",
+            "which could potentially be an f-string. The use of f-strings is preferred. "
+            "Requires Python 3.6 and ``py-version >= 3.6``.",
         ),
     }
 
+    def open(self) -> None:
+        py_version = self.linter.config.py_version
+        self._py36_plus = py_version >= (3, 6)
+
     @staticmethod
-    def _is_builtin(node, function):
+    def _is_builtin(node: nodes.NodeNG, function: str) -> bool:
         inferred = utils.safe_infer(node)
         if not inferred:
             return False
         return utils.is_builtin_object(inferred) and inferred.name == function
 
-    @utils.check_messages("consider-iterating-dictionary", "use-maxsplit-arg")
+    @utils.only_required_for_messages(
+        "consider-iterating-dictionary", "use-maxsplit-arg"
+    )
     def visit_call(self, node: nodes.Call) -> None:
         self._check_consider_iterating_dictionary(node)
         self._check_use_maxsplit_arg(node)
@@ -77,13 +85,19 @@ class RecommendationChecker(checkers.BaseChecker):
             return
         if node.func.attrname != "keys":
             return
+
+        if isinstance(node.parent, nodes.BinOp) and node.parent.op in {"&", "|", "^"}:
+            return
+
+        comp_ancestor = utils.get_node_first_ancestor_of_type(node, nodes.Compare)
         if (
             isinstance(node.parent, (nodes.For, nodes.Comprehension))
-            or isinstance(node.parent, nodes.Compare)
+            or comp_ancestor
             and any(
                 op
-                for op, comparator in node.parent.ops
-                if op == "in" and comparator is node
+                for op, comparator in comp_ancestor.ops
+                if op in {"in", "not in"}
+                and (comparator in node.node_ancestors() or comparator is node)
             )
         ):
             inferred = utils.safe_infer(node.func)
@@ -91,31 +105,43 @@ class RecommendationChecker(checkers.BaseChecker):
                 inferred.bound, nodes.Dict
             ):
                 return
-
-            self.add_message("consider-iterating-dictionary", node=node)
+            self.add_message(
+                "consider-iterating-dictionary", node=node, confidence=INFERENCE
+            )
 
     def _check_use_maxsplit_arg(self, node: nodes.Call) -> None:
-        """Add message when accessing first or last elements of a str.split() or str.rsplit()."""
-
+        """Add message when accessing first or last elements of a str.split() or
+        str.rsplit().
+        """
         # Check if call is split() or rsplit()
         if not (
             isinstance(node.func, nodes.Attribute)
-            and node.func.attrname in ("split", "rsplit")
+            and node.func.attrname in {"split", "rsplit"}
             and isinstance(utils.safe_infer(node.func), astroid.BoundMethod)
         ):
             return
-
-        try:
-            utils.get_argument_from_call(node, 0, "sep")
-        except utils.NoSuchArgumentError:
+        inferred_expr = utils.safe_infer(node.func.expr)
+        if isinstance(inferred_expr, astroid.Instance) and any(
+            inferred_expr.nodes_of_class(nodes.ClassDef)
+        ):
             return
 
+        confidence = HIGH
+        try:
+            sep = utils.get_argument_from_call(node, 0, "sep")
+        except utils.NoSuchArgumentError:
+            sep = utils.infer_kwarg_from_call(node, keyword="sep")
+            confidence = INFERENCE
+            if not sep:
+                return
+
         try:
             # Ignore if maxsplit arg has been set
             utils.get_argument_from_call(node, 1, "maxsplit")
             return
         except utils.NoSuchArgumentError:
-            pass
+            if utils.infer_kwarg_from_call(node, keyword="maxsplit"):
+                return
 
         if isinstance(node.parent, nodes.Subscript):
             try:
@@ -147,11 +173,16 @@ class RecommendationChecker(checkers.BaseChecker):
                 new_name = (
                     node.func.as_string().rsplit(fn_name, maxsplit=1)[0]
                     + new_fn
-                    + f"({node.args[0].as_string()}, maxsplit=1)[{subscript_value}]"
+                    + f"({sep.as_string()}, maxsplit=1)[{subscript_value}]"
+                )
+                self.add_message(
+                    "use-maxsplit-arg",
+                    node=node,
+                    args=(new_name,),
+                    confidence=confidence,
                 )
-                self.add_message("use-maxsplit-arg", node=node, args=(new_name,))
 
-    @utils.check_messages(
+    @utils.only_required_for_messages(
         "consider-using-enumerate",
         "consider-using-dict-items",
         "use-sequence-for-iteration",
@@ -276,11 +307,15 @@ class RecommendationChecker(checkers.BaseChecker):
                     # Ignore this subscript if it is the target of an assignment
                     # Early termination as dict index lookup is necessary
                     return
+                if isinstance(subscript.parent, nodes.Delete):
+                    # Ignore this subscript if the index is used to delete a
+                    # dictionary item.
+                    return
 
                 self.add_message("consider-using-dict-items", node=node)
                 return
 
-    @utils.check_messages(
+    @utils.only_required_for_messages(
         "consider-using-dict-items",
         "use-sequence-for-iteration",
     )
@@ -313,28 +348,38 @@ class RecommendationChecker(checkers.BaseChecker):
                 return
 
     def _check_use_sequence_for_iteration(
-        self, node: Union[nodes.For, nodes.Comprehension]
+        self, node: nodes.For | nodes.Comprehension
     ) -> None:
-        """Check if code iterates over an in-place defined set."""
-        if isinstance(node.iter, nodes.Set):
-            self.add_message("use-sequence-for-iteration", node=node.iter)
+        """Check if code iterates over an in-place defined set.
 
-    @utils.check_messages("consider-using-f-string")
-    def visit_const(self, node: nodes.Const) -> None:
-        if node.pytype() == "builtins.str" and not isinstance(
-            node.parent, nodes.JoinedStr
+        Sets using `*` are not considered in-place.
+        """
+        if isinstance(node.iter, nodes.Set) and not any(
+            utils.has_starred_node_recursive(node)
         ):
-            self._detect_replacable_format_call(node)
+            self.add_message(
+                "use-sequence-for-iteration", node=node.iter, confidence=HIGH
+            )
+
+    @utils.only_required_for_messages("consider-using-f-string")
+    def visit_const(self, node: nodes.Const) -> None:
+        if self._py36_plus:
+            # f-strings require Python 3.6
+            if node.pytype() == "builtins.str" and not isinstance(
+                node.parent, nodes.JoinedStr
+            ):
+                self._detect_replacable_format_call(node)
 
     def _detect_replacable_format_call(self, node: nodes.Const) -> None:
         """Check whether a string is used in a call to format() or '%' and whether it
-        can be replaced by a f-string"""
+        can be replaced by an f-string.
+        """
         if (
             isinstance(node.parent, nodes.Attribute)
             and node.parent.attrname == "format"
         ):
-            # Allow assigning .format to a variable
-            if isinstance(node.parent.parent, nodes.Assign):
+            # Don't warn on referencing / assigning .format without calling it
+            if not isinstance(node.parent.parent, nodes.Call):
                 return
 
             if node.parent.parent.args:
@@ -347,6 +392,9 @@ class RecommendationChecker(checkers.BaseChecker):
                             and len(inferred.elts) > 1
                         ):
                             return
+                    # Backslashes can't be in f-string expressions
+                    if "\\" in arg.as_string():
+                        return
 
             elif node.parent.parent.keywords:
                 keyword_args = [
@@ -373,6 +421,20 @@ class RecommendationChecker(checkers.BaseChecker):
             )
 
         elif isinstance(node.parent, nodes.BinOp) and node.parent.op == "%":
+            # Backslashes can't be in f-string expressions
+            if "\\" in node.parent.right.as_string():
+                return
+
+            # If % applied to another type than str, it's modulo and can't be replaced by formatting
+            if not hasattr(node.parent.left, "value") or not isinstance(
+                node.parent.left.value, str
+            ):
+                return
+
+            # Brackets can be inconvenient in f-string expressions
+            if "{" in node.parent.left.value or "}" in node.parent.left.value:
+                return
+
             inferred_right = utils.safe_infer(node.parent.right)
 
             # If dicts or lists of length > 1 are used
