@@ -1,6 +1,6 @@
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
-# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
+# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt
 
 """Functional/non regression tests for pylint."""
 
@@ -29,7 +29,7 @@ def exception_str(
     self: Exception, ex: Exception  # pylint: disable=unused-argument
 ) -> str:
     """Function used to replace default __str__ method of exception instances
-    This function is not typed because it is legacy code
+    This function is not typed because it is legacy code.
     """
     return f"in {ex.file}\n:: {', '.join(ex.args)}"  # type: ignore[attr-defined] # Defined in the caller
 
@@ -44,8 +44,7 @@ class LintTestUsingModule:
     output: str | None = None
 
     def _test_functionality(self) -> None:
-        if self.module:
-            tocheck = [self.package + "." + self.module]
+        tocheck = [self.package + "." + self.module] if self.module else []
         if self.depends:
             tocheck += [
                 self.package + f".{name.replace('.py', '')}" for name, _ in self.depends
@@ -72,7 +71,7 @@ class LintTestUsingModule:
         try:
             self.linter.check(tocheck)
         except Exception as ex:
-            print(f"Exception: {ex} in {tocheck}:: {'‚ '.join(ex.args)}")
+            print(f"Exception: {ex} in {tocheck}:: {', '.join(ex.args)}")
             # This is legacy code we're trying to remove, not worth it to type correctly
             ex.file = tocheck  # type: ignore[attr-defined]
             print(ex)
@@ -115,7 +114,7 @@ def gen_tests(
         is_to_run = re.compile(filter_rgx).search
     else:
         is_to_run = (  # noqa: E731, We're going to throw all this anyway
-            lambda x: 1  # type: ignore[assignment,misc] # pylint: disable=unnecessary-lambda-assignment
+            lambda x: 1  # type: ignore[assignment] # pylint: disable=unnecessary-lambda-assignment
         )
     tests: list[tuple[str, str, list[tuple[str, str]]]] = []
     for module_file, messages_file in _get_tests_info(INPUT_DIR, MSG_DIR, "func_", ""):
